@@ -1,4 +1,7 @@
-from distutils.version import LooseVersion
+try:
+    from packaging.version import Version
+except ImportError:
+    from distutils.version import LooseVersion as Version
 
 try:
     import pandas as pd
@@ -9,5 +12,5 @@ else:
         pv = pd._version.get_versions()['version']
     else:
         pv = pd.version.version
-    if LooseVersion(pv) < LooseVersion('0.17'):
+    if Version(pv) < Version('0.17'):
         pd.DataFrame.sort_values = pd.DataFrame.sort
