@@ -387,16 +387,17 @@ class Composition(BasicComposition):
             Examples of `charge_carrier`: "H+", "NH3+"
             (here, 3 is part of the composition, and + is a single charge),
             "Fe+2" ("Fe" is the formula and "+2" is the charge).
-            .. note ::
-                `charge` must be a multiple of `charge_carrier` charge.
+            .. note :: `charge` must be a multiple of `charge_carrier` charge.
+
             If dict, it is the atomic composition of the group.
             In this case, the charge can be passed separately as `carrier_charge`
             or it will be deduced from the number of protons in `charge_carrier`.
         carrier_charge : int, optional
             Charge of the charge carrier group (if `charge_carrier` is specified
             as a composition dict).
-            .. note ::
-                `charge` must be a multiple of `charge_charge`.
+
+            .. note :: `charge` must be a multiple of `charge_charge`.
+
         mass_data : dict, optional
             A dict with the masses of the chemical elements (the default
             value is :py:data:`nist_mass`).
@@ -590,16 +591,20 @@ def calculate_mass(*args, **kwargs):
         Examples of `charge_carrier`: "H+", "NH3+"
         (here, 3 is part of the composition, and + is a single charge),
         "Fe+2" ("Fe" is the formula and "+2" is the charge).
+
         .. note ::
             `charge` must be a multiple of `charge_carrier` charge.
+
         If dict, it is the atomic composition of the group.
         In this case, the charge can be passed separately as `carrier_charge`
         or it will be deduced from the number of protons in `charge_carrier`.
     carrier_charge : int, optional
         Charge of the charge carrier group (if `charge_carrier` is specified
         as a composition dict).
+
         .. note ::
             `charge` must be a multiple of `charge_charge`.
+
     mass_data : dict, optional
         A dict with the masses of the chemical elements (the default
         value is :py:data:`nist_mass`).
@@ -959,8 +964,6 @@ def fast_mass2(sequence, ion_type=None, charge=None, **kwargs):
     """
     aa_mass = kwargs.get('aa_mass', std_aa_mass)
     mass_data = kwargs.get('mass_data', nist_mass)
-    aa_mass.setdefault('H-', mass_data['H'][0][0])
-    aa_mass.setdefault('-OH', mass_data['H'][0][0] + mass_data['O'][0][0])
     try:
         comp = parser.amino_acid_composition(sequence,
                 show_unmodified_termini=True,
@@ -975,6 +978,9 @@ def fast_mass2(sequence, ion_type=None, charge=None, **kwargs):
         for aa, num in comp.items():
             if aa in aa_mass:
                 mass += aa_mass[aa] * num
+            elif parser.is_term_mod(aa):
+                assert num == 1
+                mass += calculate_mass(formula=aa.strip('-'), mass_data=mass_data)
             else:
                 mod, X = parser._split_label(aa)
                 mass += (aa_mass[mod] + aa_mass[X]) * num
@@ -1198,3 +1204,11 @@ class Unimod():
         return self._mods[self._id[i]]
 
     __getitem__ = by_id
+
+
+def neutral_mass(mz, z, charge_carrier=_nist_mass[PROTON][0][0]):
+    return (mz * abs(z)) - (z * charge_carrier)
+
+
+def mass_charge_ratio(neutral_mass, z, charge_carrier=_nist_mass[PROTON][0][0]):
+    return (neutral_mass + (z * charge_carrier)) / abs(z)
